
body {
  font-family: Helvetica, Arial;
  font-size: 12pt;
}

#layout_canvas {
  width: 100%;
  height: 100%;
}

input, select {
  background-color: white;
  padding: 4pt;
  font-size: 12pt;
}

.checked {
  background-color: #303030;
  color: white;
}

.unchecked {
  background-color: white;
}

div .menu-left-frame {
  position: absolute;
  left: 4pt;
  top: 4pt;
}

div .menu-right-frame {
  position: absolute;
  right: 4pt;
  top: 4pt;
}

.viewer-panel {
  overflow: auto;
  display: table;
  border-spacing: 0.5rem;
  padding: 0;
}

.viewer-panel-sub {
  display: table-row;
}

.viewer-frame-cell {
  position: relative;
  display: table-cell;
  vertical-align: top;
  border: 1px solid #000000;
  overflow: hidden;
}

.viewer-frame {
  width: 800px;
  height: 500px;
  resize: both;
  overflow: hidden;
  padding: 10px;
  padding-top: 40pt;
}

.viewer-layers-cell {
  display: table-cell;
  vertical-align: top;
  border: 1px solid #000000;
}

.viewer-layers {
  min-width: 20%;
  white-space: nowrap;
  overflow-y: scroll;
  padding: 0.5rem;
  height: 500px;
  padding: 10px;
}

.layer-name-cell {
  padding-left: 0.5rem;
}

.layer-visible-cell {
  padding-right: 0.5rem;
}

a {
  padding: 5px;
  margin: 10px;
  display: inline-block;
  border: solid;
  border-width: 2px;
  color: red;
}
